function Graph(canvas) {
  this.canvas = canvas;
  this.context = canvas.getContext('2d');
  this.sparkContext = document.getElementById('sparkCanvas').getContext('2d');
  this.nodes = {};
  this.edges = {};
  
  this.physics = {
    gravity: 1000,
    minEntropy: 1,
    springConstant: 0.1,
    originConstant: 0.01,
    maxForce: 25,
    springRest: 30
  };
  
  this.nodeRadius = 8;
  
  this.maxSteps = 200;
  
  this.origin = {x: 600, y: 200};
  
  this.padding = {x: 10, y: 10};
  
  this.width = 1000;
  this.height = 650;
  
}


Graph.prototype = {
  addNode: function(identity) {
    if (this.nodes[identity]) {
      throw new Error("Already added a node with the identity" + identity)
    } else {
      this.nodes[identity] = new Graph.Node(identity, this.nodeRadius);
      return this.nodes[identity];
    }
  },
  get: function(identity) {
    return this.nodes[identity];
  },
  addEdge: function(nodeA, nodeB) {
    if (!nodeA.connectedTo(nodeB)) {
      this.edges[Graph.Edge.keyFor(nodeA,nodeB)] = new Graph.Edge(nodeA, nodeB);
    }
  },
  removeEdge: function(nodeA, nodeB) {
    if (nodeA.connectedTo(nodeB)) {
      var key = Graph.Edge.keyFor(nodeA,nodeB);
      this.edges[key].remove();
      delete this.edges[key];
    }
  },
  draw: function() {
    this.context.clearRect(0, 0, 1200, 1000);
    this.sparkContext.clearRect(0, 0, 1200, 1000);

    for (var i in this.edges) {
      this.edges[i].draw(this.context, this.sparkContext);
    }
    
    for (var i in this.nodes) {
      this.nodes[i].draw(this.context);
    }
  },
  step: function(i) {
    if (i > 0) {
      this.draw();
      var that = this;
      setTimeout(function() {
        that.step(i - 1)
      }, 0);
    }
  },
  applyForces: function() {
    var nodeA, nodeB, distance, magnitude, force, totalEntropy = 0;
    
    for (var i in this.nodes) {
      force = {x: 0, y: 0};
      nodeA = this.nodes[i];
      
      // Repulsive forces
      for (var j in this.nodes) {
        nodeB = this.nodes[j];
        if (nodeA != nodeB) {
          distance = nodeA.distanceTo(nodeB);
          magnitude = this.physics.gravity * (((nodeA.neighborSize+1) * (nodeB.neighborSize+1)) / Math.pow(distance, 2));
          force.x += ((nodeA.x - nodeB.x) / distance) * magnitude;
          force.y += ((nodeA.y - nodeB.y) / distance) * magnitude;
        }
      }
      
      // Attractive forces
      for (var j in nodeA.edges) {
        nodeB = nodeA.edges[j].otherNode(nodeA);
      
        distance = nodeA.distanceTo(nodeB) ;
        magnitude = -this.physics.springConstant * (distance - this.physics.springRest);
        force.x += ((nodeA.x - nodeB.x) / distance) * magnitude;
        force.y += ((nodeA.y - nodeB.y) / distance) * magnitude;
      }

      // Origin Forces
      distance = Math.sqrt(Math.pow(nodeA.x - this.origin.x, 2) + Math.pow(nodeA.y - this.origin.y, 2));
      magnitude = -1 * this.physics.originConstant * distance;
      force.x += ((nodeA.x - this.origin.x) / distance) * magnitude;
      force.y += ((nodeA.y - this.origin.y) / distance) * magnitude;
      
      // Adjust Position
      force.x = Math.max(Math.min(force.x, this.physics.maxForce), -this.physics.maxForce);
      force.y = Math.max(Math.min(force.y, this.physics.maxForce), -this.physics.maxForce);
      
      nodeA.x = Math.max(nodeA.radius + this.padding.x, Math.min(this.width - nodeA.radius - this.padding.x, nodeA.x + force.x));
      nodeA.y = Math.max(nodeA.radius + this.padding.y, Math.min(this.height - nodeA.radius - this.padding.y, nodeA.y + force.y));
              
      totalEntropy += Math.sqrt(Math.pow(force.x, 2) + Math.pow(force.y, 2));
    }
    
    return totalEntropy;
  },
  forceDirect: function() {
    var steps = 0;
    if (!true) {
      var entropy = this.applyForces();
      while (entropy > this.physics.minEntropy && steps < this.maxSteps) {
        entropy = this.applyForces();
        steps++;
      }
      
      this.draw();
    } else {
      var that = this;
      var go = function() {
        var entropy = that.applyForces();
        if (entropy > that.physics.minEntropy && steps < that.maxSteps) {
          steps++;
          that.draw();
          setTimeout(go, 0);
        }
      };
      go();
    }
  }
};

Graph.Node = function(identity, radius) {
  this.identity = identity;
  this.x = Math.random() * 50 + 600;
  this.y = Math.random() * 50 + 300;
  this.radius = radius;
  this.edges = {};
  this.neighborSize = 0;
};

Graph.Node.compare = function(lhs, rhs) {
  return lhs < rhs ? -1 : (lhs == rhs ? 0 : 1);
}

Graph.Node.prototype = {
  draw: function(context) {
    context.beginPath();
    context.fillStyle = '#444';
    context.shadowOffsetX = 2;
    context.shadowOffsetY = 2;
    context.shadowBlur = 5;
    context.arc(this.x, this.y, this.radius, Math.PI * 2, false);
    context.fill();
    
    context.strokeStyle = '#fff'
    context.lineWidth = 1;
    context.shadowOffsetY = context.shadowOffsetX = context.shadowBlur = 0;
    context.stroke();
  },
  distanceTo: function(node) {
    return Math.sqrt(Math.pow(this.x - node.x, 2) + Math.pow(this.y - node.y, 2));
  },
  spark: function() {
    for(var i = 0; i < this.edges.length; ++i) {
      this.edges[i].spark(this);
    }
  },
  connectedTo: function(other) {
    return !! this.edges[Graph.Edge.keyFor(this, other)];
  },
  neighbors: function() {
    var neighbors = [];
    for (var i in this.edges) {
      neighbors.push(this.edges[i].otherNode(this));
    }
    return neighbors;
  },
  toString: function() {
    return this.identity;
  }
};

Graph.Edge = function(nodeA, nodeB) {
  this.key = Graph.Edge.keyFor(nodeA,nodeB);
  this.nodeA = nodeA;
  this.nodeB = nodeB;
  this.nodeA.edges[this.key] = this;
  this.nodeA.neighborSize++;
  this.nodeB.edges[this.key] = this;
  this.nodeB.neighborSize++;
};

Graph.Edge.keyFor = function(nodeA, nodeB) {
  return [nodeA, nodeB].sort(Graph.Node.compare).join("");
}

Graph.Edge.prototype = {
  draw: function(context, sparkContext) {
    context.beginPath();
    context.strokeStyle = '#aaa';
    context.lineWidth = 1;
    context.shadowOffsetY = context.shadowOffsetX = context.shadowBlur = 0;
    context.moveTo(this.nodeA.x, this.nodeA.y);
    context.lineTo(this.nodeB.x, this.nodeB.y);
    context.stroke();
    if (this.sparks) {
      sparkContext.beginPath();
      sparkContext.strokeStyle = '#000';
      sparkContext.lineWidth = 1;
      sparkContext.moveTo(this.sparks[this.sparkIndex].x, this.sparks[this.sparkIndex].y);
      sparkContext.lineTo(this.sparks[this.sparkIndex + 1].x, this.sparks[this.sparkIndex + 1].y);
      sparkContext.stroke();
      
      this.sparkIndex++;
      if (this.sparkIndex == this.sparks.length - 1) {
        this.sparks = null;
      }
    }
  },
  spark: function(starting) {
    this.sparkIndex = 0;
    this.sparks = [];
    var startX = starting.x;
    var startY = starting.y;
    
    var other = this.nodeA == starting ? this.nodeB : this.nodeA;
    var endX = other.x, endY = other.y;

    var distance = starting.distanceTo(other);
    
    this.sparkSize = distance / 30;
    
    var stepX = (endX - startX) / 30;
    var stepY = (endY - startY) / 30;
    
    
    for (var i = 0; i < 30; ++i) {
      this.sparks.push({x: (startX + (stepX * i)), y: (startY + (stepY * i))});
    }
  },
  otherNode: function(thisEdge) {
    return thisEdge == this.nodeA ? this.nodeB : this.nodeA;
  },
  remove: function() {
    delete this.nodeA.edges[this.key];
    this.nodeA.neighborSize--;
    delete this.nodeB.edges[this.key];
    this.nodeB.neighborSize--;
  }
};

